@@ -60,7 +60,7 @@ def determine_config_file():
 def load_config_file(config, path):
     with open(path) as f:
         text = f.read()
-        config.update(yaml.load(text))
+        config.update(yaml.load(text) or {})
 
 
 def load_env_vars(config):
@@ -81,16 +81,17 @@ def _initialize_logging_old_style(config):
             }
         }
     """
-    loggers = config.get('logging', {})
-    loggers.setdefault('distributed', 'info')
-    # We could remove those lines and let the default config.yaml handle it
-    loggers.setdefault('tornado', 'critical')
-    loggers.setdefault('tornado.application', 'error')
-
-    fmt = '%(name)s - %(levelname)s - %(message)s'
+    loggers = {  # default values
+        'distributed': 'info',
+        'distributed.client': 'warning',
+        'bokeh': 'critical',
+        'tornado': 'critical',
+        'tornado.application': 'error',
+    }
+    loggers.update(config.get('logging', {}))
 
     handler = logging.StreamHandler(sys.stderr)
-    handler.setFormatter(logging.Formatter(fmt))
+    handler.setFormatter(logging.Formatter(log_format))
     for name, level in loggers.items():
         if isinstance(level, str):
             level = logging_names[level.upper()]
@@ -133,7 +134,9 @@ def initialize_logging(config):
 
 
 @contextmanager
-def set_config(**kwargs):
+def set_config(arg=None, **kwargs):
+    if arg and not kwargs:
+        kwargs = arg
     old = {}
     for key in kwargs:
         if key in config:
@@ -162,4 +165,6 @@ else:
 
 load_env_vars(config)
 
+log_format = config.get('log-format', '%(name)s - %(levelname)s - %(message)s')
+
 initialize_logging(config)
