@@ -7,6 +7,7 @@ import os
 import psutil
 import shutil
 import threading
+import uuid
 
 from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
@@ -21,7 +22,7 @@ from .process import AsyncProcess
 from .proctitle import enable_proctitle_on_children
 from .security import Security
 from .utils import (get_ip, mp_context, silence_logging, json_load_robust,
-        PeriodicCallback)
+        PeriodicCallback, parse_timedelta)
 from .worker import _ncores, run, parse_memory_limit
 
 
@@ -153,9 +154,11 @@ class Nanny(ServerNode):
 
         logger.info('        Start Nanny at: %r', self.address)
         response = yield self.instantiate()
-        if response == 'OK':
+        if response == 'running':
             assert self.worker_address
             self.status = 'running'
+        else:
+            yield self._close()
 
         self.start_periodic_callbacks()
 
@@ -216,14 +219,16 @@ class Nanny(ServerNode):
         self.auto_restart = True
         if self.death_timeout:
             try:
-                yield gen.with_timeout(timedelta(seconds=self.death_timeout),
-                                       self.process.start())
+                result = yield gen.with_timeout(
+                        timedelta(seconds=self.death_timeout),
+                        self.process.start()
+                )
             except gen.TimeoutError:
                 yield self._close(timeout=self.death_timeout)
                 raise gen.Return('timed out')
         else:
-            yield self.process.start()
-        raise gen.Return('OK')
+            result = yield self.process.start()
+        raise gen.Return(result)
 
     @gen.coroutine
     def restart(self, comm=None, timeout=2, executor_wait=True):
@@ -247,7 +252,11 @@ class Nanny(ServerNode):
         """ Track worker's memory.  Restart if it goes above 95% """
         if self.status != 'running':
             return
-        memory = psutil.Process(self.process.pid).memory_info().rss
+        try:
+            proc = psutil.Process(self.process.pid)
+        except psutil.NoSuchProcess:
+            return
+        memory = proc.memory_info().rss
         frac = memory / self.memory_limit
         if self.memory_terminate_fraction and frac > self.memory_terminate_fraction:
             logger.warning("Worker exceeded 95% memory budget.  Restarting")
@@ -327,17 +336,18 @@ class WorkerProcess(object):
         """
         enable_proctitle_on_children()
         if self.status == 'running':
-            return
+            raise gen.Return(self.status)
         if self.status == 'starting':
             yield self.running.wait()
-            return
+            raise gen.Return(self.status)
 
         while True:
-            # FIXME: this sometimes stalls in _wait_until_running
+            # FIXME: this sometimes stalls in _wait_until_connected
             # our temporary solution is to retry a few times if the process
             # doesn't start up in five seconds
-            self.init_result_q = mp_context.Queue()
+            self.init_result_q = init_q = mp_context.Queue()
             self.child_stop_q = mp_context.Queue()
+            uid = uuid.uuid4().hex
             try:
                 self.process = AsyncProcess(
                     target=self._run,
@@ -346,7 +356,8 @@ class WorkerProcess(object):
                                 worker_start_args=self.worker_start_args,
                                 silence_logs=self.silence_logs,
                                 init_result_q=self.init_result_q,
-                                child_stop_q=self.child_stop_q),
+                                child_stop_q=self.child_stop_q,
+                                uid=uid),
                 )
                 self.process.daemon = True
                 self.process.set_exit_callback(self._on_exit)
@@ -355,8 +366,9 @@ class WorkerProcess(object):
                 self.status = 'starting'
                 yield self.process.start()
                 if self.status == 'starting':
-                    yield gen.with_timeout(timedelta(seconds=5),
-                                           self._wait_until_running())
+                    timeout = parse_timedelta(config.get('nanny-start-timeout', '30s'))
+                    yield gen.with_timeout(timedelta(seconds=timeout),
+                                           self._wait_until_started(uid))
             except gen.TimeoutError:
                 logger.info("Failed to start worker process.  Restarting")
                 yield gen.with_timeout(timedelta(seconds=1),
@@ -364,6 +376,20 @@ class WorkerProcess(object):
             else:
                 break
 
+        if self.status == 'starting':
+            msg = yield self._wait_until_connected(uid)
+            if not msg:
+                raise gen.Return(self.status)
+            self.worker_address = msg['address']
+            self.worker_dir = msg['dir']
+            assert self.worker_address
+            self.status = 'running'
+            self.running.set()
+
+        init_q.close()
+
+        raise gen.Return(self.status)
+
     def _on_exit(self, proc):
         if proc is not self.process:
             # Ignore exit of old process instance
@@ -428,10 +454,12 @@ class WorkerProcess(object):
         self.status = 'stopping'
 
         process = self.process
-        self.child_stop_q.put({'op': 'stop',
-                               'timeout': max(0, deadline - loop.time()) * 0.8,
-                               'executor_wait': executor_wait,
-                               })
+        self.child_stop_q.put({
+            'op': 'stop',
+            'timeout': max(0, deadline - loop.time()) * 0.8,
+            'executor_wait': executor_wait,
+        })
+        self.child_stop_q.close()
 
         while process.is_alive() and loop.time() < deadline:
             yield gen.sleep(0.05)
@@ -445,7 +473,26 @@ class WorkerProcess(object):
                 logger.error("Failed to kill worker process: %s", e)
 
     @gen.coroutine
-    def _wait_until_running(self):
+    def _wait_until_started(self, uid):
+        delay = 0.05
+        while True:
+            if self.status != 'starting':
+                return
+            try:
+                msg = self.init_result_q.get_nowait()
+                if msg['uid'] != uid:  # ensure that we didn't cross queues
+                    continue
+                if msg['status'] != 'started':
+                    logger.warn("Nanny got unexpected message %s. "
+                                "Starting worker again", msg)
+                    raise gen.TimeoutError()
+                return
+            except Empty:
+                yield gen.sleep(delay)
+                continue
+
+    @gen.coroutine
+    def _wait_until_connected(self, uid):
         delay = 0.05
         while True:
             if self.status != 'starting':
@@ -456,20 +503,20 @@ class WorkerProcess(object):
                 yield gen.sleep(delay)
                 continue
 
-            if isinstance(msg, Exception):
+            if msg['uid'] != uid:  # ensure that we didn't cross queues
+                continue
+
+            if 'exception' in msg:
+                logger.error("Failed while trying to start worker process: %s",
+                             msg['exception'])
                 yield self.process.join()
                 raise msg
             else:
-                self.worker_address = msg['address']
-                self.worker_dir = msg['dir']
-                assert self.worker_address
-                self.status = 'running'
-                self.running.set()
                 raise gen.Return(msg)
 
     @classmethod
     def _run(cls, worker_args, worker_kwargs, worker_start_args,
-             silence_logs, init_result_q, child_stop_q):  # pragma: no cover
+             silence_logs, init_result_q, child_stop_q, uid):  # pragma: no cover
         from distributed import Worker
 
         try:
@@ -508,6 +555,7 @@ class WorkerProcess(object):
                 except Empty:
                     pass
                 else:
+                    child_stop_q.close()
                     assert msg.pop('op') == 'stop'
                     loop.add_callback(do_stop, **msg)
                     break
@@ -521,15 +569,19 @@ class WorkerProcess(object):
             """
             Try to start worker and inform parent of outcome.
             """
+            init_result_q.put({'uid': uid, 'status': 'started'})
             try:
                 yield worker._start(*worker_start_args)
             except Exception as e:
                 logger.exception("Failed to start worker")
-                init_result_q.put(e)
+                init_result_q.put({'uid': uid, 'exception': e})
+                init_result_q.close()
             else:
                 assert worker.address
                 init_result_q.put({'address': worker.address,
-                                   'dir': worker.local_dir})
+                                   'dir': worker.local_dir,
+                                   'uid': uid})
+                init_result_q.close()
                 yield worker.wait_until_closed()
                 logger.info("Worker closed")
 
